import re
from colormath.color_objects import AdobeRGBColor
from colormath.color_objects import HSVColor
from colormath.color_objects import HSLColor
from colormath.color_conversions import convert_color

class Colour:
    def __init__(self, r=0, g=0, b=0):
        self.set_rgb(r, g, b)

    def __repr__(self):
        r, g, b = self.rgb

        return 'Colour({}, {}, {})'.format(r, g, b)

    def set_rgb(self, r, g, b):
        self._set_rgb_color(AdobeRGBColor(r, g, b, True))

    def set_r(self, r):
        self._rgb_color.rgb_r = r
        self._update_hsv_from_rgb()

    def set_g(self, g):
        self._rgb_color.rgb_g = g
        self._update_hsv_from_rgb()

    def set_b(self, b):
        self._rgb_color.rgb_b = b
        self._update_hsv_from_rgb()

    def set_hsv(self, h, s, v):
        self._set_hsv_color(HSVColor(h, s, v))

    def _update_rgb_from_hsv(self):
        self._rgb_color = convert_color(self._hsv_color, AdobeRGBColor)

    def _update_hsv_from_rgb(self):
        self._hsv_color = convert_color(self._rgb_color, HSVColor)

    @property
    def rgb(self):
        return self._rgb_color.get_upscaled_value_tuple()

    @property
    def hsv(self):
        c = self._hsv_color

        return c.hsv_h, c.hsv_s, c.hsv_v

    @property
    def hex(self):
        r, g, b = self.rgb

        return '{:02x}{:02x}{:02x}'.format(r, g, b)

    def set_from_hex(self, hex_str):
        hex_str = hex_str.strip()

        if hex_str.startswith('#'):
            hex_str = hex_str[1:]

        if len(hex_str) == 3:
            hex_str = '{r}{r}{g}{g}{b}{b}'.format(r=hex_str[0],
                                                  g=hex_str[1],
                                                  b=hex_str[2])

        if len(hex_str) != 6:
            return

        color = AdobeRGBColor.new_from_rgb_hex(hex_str)
        self._set_rgb_color(color)

    def _set_hsv_color(self, hsv):
        self._hsv_color = hsv
        self._update_rgb_from_hsv()

    def _set_rgb_color(self, rgb):
        self._rgb_color = rgb
        self._update_hsv_from_rgb()

    @property
    def _hsl_color(self):
        return convert_color(self._hsv_color, HSLColor)

    def inc_light(self, val):
        hsl = self._hsl_color
        light = hsl.hsl_l
        light += val

        # clip
        if light > 1:
            light = 1

        hsl.hsl_l = light
        self._set_hsv_color(convert_color(hsl, HSVColor))

    def dec_light(self, val):
        hsl = self._hsl_color
        light = hsl.hsl_l
        light -= val

        # clip
        if light < 0:
            light = 0

        hsl.hsl_l = light
        self._set_hsv_color(convert_color(hsl, HSVColor))

    def inc_sat(self, val):
        sat = self._hsv_color.hsv_s
        sat += val

        # clip
        if sat > 1:
            sat = 1

        self._hsv_color.hsv_s = sat
        self._update_rgb_from_hsv()

    def dec_sat(self, val):
        sat = self._hsv_color.hsv_s
        sat -= val

        # clip
        if sat < 0:
            sat = 0

        self._hsv_color.hsv_s = sat
        self._update_rgb_from_hsv()

    @staticmethod
    def _rgb_in_range(r, g, b):
        good = True
        for val in [r, g, b]:
            good &= (val >= 0 and val <= 255)

        return good

    def set_from_css_rgb(self, rgb_str):
        rgb_str = rgb_str.strip()
        m = re.match(r'^rgb\s*\(\s*(\d+)\s*\,\s*(\d+)\s*\,\s*(\d+)\s*\)$', rgb_str)

        if not m:
            return

        r = int(m.group(1))
        g = int(m.group(2))
        b = int(m.group(3))

        if not Colour._rgb_in_range(r, g, b):
            return

        self.set_rgb(r, g, b)

    def get_css_rgb(self):
        r, g, b = self.rgb

        return 'rgb({}, {}, {})'.format(r, g, b)

    @staticmethod
    def from_rgb(r, g, b):
        c = Colour()
        c.set_rgb(r, g, b)

        return c

    @staticmethod
    def from_hsv(h, s, v):
        c = Colour()
        c.set_hsv(h, s, v)

        return c

    @staticmethod
    def from_hex(hex_str):
        c = Colour()
        c.set_from_hex(hex_str)

        return c

    @staticmethod
    def from_css_rgb(rgb_str):
        c = Colour()
        c.set_from_css_rgb(rgb_str)

        return c

    def copy(self):
        r, g, b = self.rgb

        return Colour.from_rgb(r, g, b)



__version__ = '1.1.0'


def _split_version_suffix():
    return __version__.split('-')


def get_version_tuple():
    version, suffix = _split_version_suffix()
    parts = version.split('.')

    return (int(parts[0]), int(parts[1]), int(parts[2]))


def get_version_suffix():
    return _split_version_suffix()[1]



import math
from gi.repository import Gtk, Gdk
from delicolour.colour import Colour


class BigColour(Gtk.DrawingArea):
    def __init__(self, border_radius=4, height=75):
        # initial colours
        self._colour1 = Colour.from_rgb(0, 0, 0)
        self._colour2 = Colour.from_rgb(0, 0, 0)
        self._sel = 1

        # save border radius
        self._border_radius = border_radius

        # initialize drawing area
        super().__init__()
        self.set_size_request(0, height)
        self.connect('draw', self._on_draw)

        # initialize click action
        self.add_events(Gdk.EventMask.BUTTON_PRESS_MASK)
        self.connect('button-press-event', self._on_click)

        # user events
        self._user_on_sel_change = None

    def _get_size(self):
        alloc = self.get_allocation()

        return alloc.width, alloc.height

    def _notify_on_sel_change(self, sel):
        if self._user_on_sel_change:
            self._user_on_sel_change(sel)

    def _on_click(self, widget, event):
        x = int(event.x)
        width, height = self._get_size()

        if x < round(width / 2):
            self._set_sel(1)
        else:
            self._set_sel(2)

        self._notify_on_sel_change(self._sel)

    def _on_draw(self, drawing_area, cr):
        def do_rect_path():
            # rectangle with rounded corners
            cr.arc(width - br - lw, br + lw, br, -0.5 * math.pi, 0)
            cr.arc(width - br - lw, height - br - lw, br, 0, 0.5 * math.pi)
            cr.arc(br + lw, height - br - lw, br, 0.5 * math.pi, 1 * math.pi)
            cr.arc(br + lw, br + lw, br, 1 * math.pi, -0.5 * math.pi)
            cr.close_path()

        def do_half_path():
            cr.arc(width - br - lw, br + lw, br, -0.5 * math.pi, 0)
            cr.arc(width - br - lw, height - br - lw, br, 0, 0.5 * math.pi)
            cr.line_to(round(width / lw) - diag_dist, height - lw)
            cr.line_to(round(width / lw) + diag_dist, lw)
            cr.close_path()

        def set_source_rgb(colour):
            r, g, b = colour.rgb
            r /= 255
            g /= 255
            b /= 255
            cr.set_source_rgb(r, g, b)

        def do_dot_path(x, y):
            cr.save()
            cr.translate(x, y)
            cr.scale(dot_rad, dot_rad);
            cr.arc(0, 0, 1, 0, 2 * math.pi)
            cr.set_source_rgb(0, 0, 0)
            cr.restore()

        def draw_sel_dot():
            tr_y = dot_dist

            if self._sel == 1:
                tr_x = dot_dist + lw
            else:
                tr_x = width - dot_dist - lw

            do_dot_path(tr_x, tr_y)
            cr.fill()
            do_dot_path(tr_x, tr_y)
            cr.set_line_width(1)
            cr.set_source_rgb(1, 1, 1)
            cr.stroke()

        width, height = self._get_size()
        br = self._border_radius
        lw = 2
        dot_rad = 3
        dot_dist = 10
        diag_dist = 5

        # fill colour 1
        set_source_rgb(self._colour1)
        do_rect_path()
        cr.fill()

        # fill colour 2
        set_source_rgb(self._colour2)
        do_half_path()
        cr.fill()

        # stroke around everything
        cr.set_line_width(lw)
        cr.set_source_rgb(0.3, 0.3, 0.3)
        do_rect_path()
        cr.stroke()

        # draw selrent dot
        draw_sel_dot()

        return False

    def set_colour1(self, colour):
        self._colour1 = colour
        self.queue_draw()

    def set_colour2(self, colour):
        self._colour2 = colour
        self.queue_draw()

    def set_rgb1(self, r, g, b):
        self._colour1.set_rgb(r, g, b)
        self.queue_draw()

    def set_rgb2(self, r, g, b):
        self._colour2.set_rgb(r, g, b)
        self.queue_draw()

    def set_sel(self, sel):
        self._set_sel(sel)

    def _set_sel(self, sel):
        self._sel = sel
        self.queue_draw()

    def on_sel_change(self, cb):
        self._user_on_sel_change = cb



import gi


# required GI versions/features
gi.require_version('Gtk', '3.0')


from delicolour import main_window
from delicolour import config
from gi.repository import Gtk
import delicolour
import argparse
import warnings
import signal
import sys


def _perror(msg):
    print('Error: ' + str(msg), file=sys.stderr)
    sys.exit(1)


def _parse_args():
    ap = argparse.ArgumentParser()

    ap.add_argument('-f', '--fav-colours-count', action='store', type=int,
                    metavar='COUNT', default=12,
                    help='number of fav colours per row (default: 12)')
    ap.add_argument('-F', '--fav-colours-rows-count', action='store', type=int,
                    metavar='COUNT', default=1,
                    help='number of rows of fav colours (default: 1)')
    ap.add_argument('-i', '--increment', action='store', metavar='VALUE',
                    type=int, default=1,
                    help='fine-tuning increment (default: 1)')
    ap.add_argument('-l', '--left-colour', action='store', metavar='HEX',
                    help='initial left color (CSS hex)')
    ap.add_argument('-r', '--right-colour', action='store', metavar='HEX',
                    help='initial right color (CSS hex)')
    ap.add_argument('-V', '--version', action='version',
                    version='%(prog)s v{}'.format(delicolour.__version__))

    # parse args
    args = ap.parse_args()

    # validate --fav-colours-count
    if args.fav_colours_count < 1 or args.fav_colours_count > 16:
        _perror('--fav-colours-count must be in [1, 16] range')

    # validate --fav-colours-rows-count
    if args.fav_colours_rows_count < 1 or args.fav_colours_rows_count > 4:
        _perror('--fav-colours-rows-count must be in [1, 4] range')

    # validate --increment
    mn = config.INCR_SPINNER_MIN_VAL
    mx =  config.INCR_SPINNER_MAX_VAL

    if args.increment < mn or args.increment > mx:
        _perror('--increment must be in [{}, {}] range'.format(mn, mx))

    return args


def run():
    # enable SIGINT (Ctrl+C)
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # see <https://bugzilla.gnome.org/show_bug.cgi?id=708676#c4>
    warnings.filterwarnings('ignore', '.*g_value_get_int.*', Warning)

    # main app
    args = _parse_args()
    win = main_window.MainWindow(args)
    win.connect('delete-event', Gtk.main_quit)
    win.show_all()
    Gtk.main()



import math
from gi.repository import Gtk, Gdk
from delicolour.colour import Colour


class FavColour(Gtk.DrawingArea):
    def __init__(self, width, height):
        # initial colour
        self._colour = Colour.from_rgb(255, 255, 255)

        # initialize drawing area
        super().__init__()
        self.set_size_request(width, height)
        self.connect('draw', self._on_draw)

        # initialize click action
        self.add_events(Gdk.EventMask.BUTTON_PRESS_MASK)
        self.connect('button-press-event', self._on_click)

        # user events
        self._user_on_click = None

    def _get_size(self):
        alloc = self.get_allocation()

        return alloc.width, alloc.height

    def _notify_on_click(self, event):
        if self._user_on_click:
            self._user_on_click(event, self)

    def _on_click(self, widget, event):
        self._notify_on_click(event)

    def _on_draw(self, drawing_area, cr):
        def do_path():
            cr.rectangle(lw, lw, width - 2 * lw, height - 2 * lw)

        def set_source_rgb(colour):
            r, g, b = colour.rgb
            r /= 255
            g /= 255
            b /= 255
            cr.set_source_rgb(r, g, b)

        width, height = self._get_size()
        lw = 2

        # fill colour 1
        set_source_rgb(self._colour)
        do_path()
        cr.fill()

        # stroke around everything
        cr.set_line_width(lw)
        cr.set_source_rgb(.3, .3, .3)
        do_path()
        cr.stroke()

        return False

    def set_colour(self, colour):
        self._colour = colour
        self.queue_draw()

    @property
    def colour(self):
        return self._colour

    def on_click(self, cb):
        self._user_on_click = cb



from delicolour.colour import Colour


class AppModel:
    def __init__(self):
        self._sel = 1

    @property
    def colour(self):
        if self.sel == 1:
            return self.colour1
        else:
            return self.colour2

    @colour.setter
    def colour(self, colour):
        copy = colour.copy()

        if self.sel == 1:
            self.colour1 = copy
        else:
            self.colour2 = copy

    @property
    def sel(self):
        return self._sel

    @sel.setter
    def sel(self, sel):
        self._sel = sel

    @staticmethod
    def get_default():
        model = AppModel()
        model.colour1 = Colour.from_rgb(255, 255, 255)
        model.colour2 = Colour.from_rgb(0, 0, 0)
        model.css_hex_copy_hash = True
        model.css_hex_lower = True
        model.fine_incr = 1

        return model




from delicolour import config
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Pango


class AdjustmentControls(Gtk.Box):
    def __init__(self, minval, maxval, init_adj):
        # user callbacks
        self._user_on_incr_change = None

        # parameters
        self._minval = minval
        self._maxval = maxval

        # parent box
        super().__init__(spacing=0, homogeneous=False)
        self.set_orientation(Gtk.Orientation.HORIZONTAL)

        # controls
        self._init_controls()
        self._value_lbl.set_text(str(init_adj))

    @staticmethod
    def _new_btn(label, tooltip_text):
        btn = Gtk.Button(label=label)
        btn.set_tooltip_text(tooltip_text)
        btn.set_size_request(30, -1)
        btn.set_valign(Gtk.Align.CENTER)
        my_style_provider = Gtk.CssProvider()
        my_style_provider.load_from_data(b'GtkWidget {padding: 3px;}')
        style_context = btn.get_style_context()
        style_context.add_provider(my_style_provider,
                                   Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

        return btn

    def _init_controls(self):
        # label
        self._value_lbl = Gtk.Label()
        self._value_lbl.modify_font(Pango.FontDescription('sans-serif bold 8'))
        self._value_lbl.set_text(str(self._minval))
        self._value_lbl.set_width_chars(2)
        self._value_lbl.set_alignment(0, 0.5)
        self._value_lbl.set_tooltip_text('Fine-tuning increment')

        # buttons
        self._dec_btn = self._new_btn('-', 'Decrement fine-tuning increment')
        self._inc_btn = self._new_btn('+', 'Increment fine-tuning increment')

        # set callbacks
        self._dec_btn.connect('clicked', self._on_dec)
        self._inc_btn.connect('clicked', self._on_inc)

        # alignments
        a_pad_left = Gtk.Alignment()
        a_pad_left.set_padding(0, 0, config.MAIN_GUTTER_PX, 0)
        a_pad_left.add(self._dec_btn)

        # pack into box
        self.pack_start(self._value_lbl, False, False, 0)
        self.pack_start(a_pad_left, False, False, 0)
        self.pack_start(self._inc_btn, False, False, 0)

    def _on_incr_change(self):
        if self._user_on_incr_change:
            self._user_on_incr_change()

    def _on_inc(self, btn):
        val = self.value

        if val == self._maxval:
            return

        val += 1
        self._value_lbl.set_text(str(val))
        self._on_incr_change()

    def _on_dec(self, btn):
        val = self.value

        if val == self._minval:
            return

        val -= 1
        self._value_lbl.set_text(str(val))
        self._on_incr_change()

    @property
    def value(self):
        return int(self._value_lbl.get_text())

    def on_incr_change(self, cb):
        self._user_on_incr_change = cb



from delicolour.fav_colour import FavColour
from delicolour.colour import Colour
from gi.repository import Gtk, Gdk
from delicolour import config
import math


class FavColours(Gtk.Box):
    def __init__(self, max_height=(3 * config.MAIN_GUTTER_PX),
                 fav_colours_count=config.DEF_FAV_COLOURS_COUNT):
        super().__init__(spacing=config.MAIN_GUTTER_PX)
        self._fav_colours_count = fav_colours_count
        self._max_height = max_height
        self.connect('realize', self._on_realize)
        self._user_on_fav_colour_click = None

    def _get_size(self):
        alloc = self.get_allocation()

        return alloc.width, alloc.height

    def _on_realize(self, widget):
        width, height = self._get_size()
        colours_count = self._fav_colours_count
        self._fav_colours = []

        # remove gutters from total width
        width -= (colours_count - 1) * config.MAIN_GUTTER_PX

        # divide remaining width equally
        fav_colour_width = round(width / colours_count)

        # clip height if needed
        fav_colour_height = fav_colour_width

        if fav_colour_height > self._max_height:
            fav_colour_height = self._max_height

        for c in range(colours_count):
            fav_colour = FavColour(fav_colour_width, fav_colour_height)
            fav_colour.on_click(self._notify_on_fav_colour_click)
            self._fav_colours.append(fav_colour)
            self.pack_start(fav_colour, False, False, 0)
            fav_colour.show()

    @property
    def fav_colours(self):
        return self._fav_colours

    def _notify_on_fav_colour_click(self, event, fav_colour):
        if self._user_on_fav_colour_click:
            self._user_on_fav_colour_click(event, fav_colour)

    def on_fav_colour_click(self, cb):
        self._user_on_fav_colour_click = cb



import re
from delicolour import config
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Pango
from pkg_resources import resource_filename
from delicolour.adjustment_controls import AdjustmentControls


class FineControls(Gtk.Box):
    def __init__(self, minval, maxval, init_adj):
        # parameters
        self._minval = minval
        self._maxval = maxval
        self._user_on_inc_light = None
        self._user_on_dec_light = None
        self._user_on_inc_sat = None
        self._user_on_dec_sat = None

        # parent box
        super().__init__(spacing=0, homogeneous=False)
        self.set_orientation(Gtk.Orientation.HORIZONTAL)

        # controls
        self._init_controls(init_adj)

    @staticmethod
    def _new_fine_control_btn(img_filename, tooltip_text):
        img = Gtk.Image()
        img.set_from_file(img_filename)
        btn = Gtk.Button()
        btn.set_tooltip_text(tooltip_text)
        btn.add(img)

        return btn

    @staticmethod
    def _get_filename(img):
        filename = 'res/{}.png'.format(img)

        return resource_filename(__name__, filename)

    def _init_controls(self, init_adj):
        new = FineControls._new_fine_control_btn
        gfn = self._get_filename

        # buttons
        self._inc_light = new(gfn('sun-24'), 'Increase lightness')
        self._dec_light = new(gfn('moon-24'), 'Decrease lightness')
        self._inc_sat = new(gfn('sat-24'), 'Increase saturation')
        self._dec_sat = new(gfn('desat-24'), 'Decrease saturation')

        # set callbacks
        self._inc_light.connect('clicked', self._on_inc_light_clicked)
        self._dec_light.connect('clicked', self._on_dec_light_clicked)
        self._inc_sat.connect('clicked', self._on_inc_sat_clicked)
        self._dec_sat.connect('clicked', self._on_dec_sat_clicked)

        # alignments
        a_pad_left = Gtk.Alignment()
        a_pad_left.set_padding(0, 0, config.MAIN_GUTTER_PX, 0)
        a_pad_left.add(self._dec_light)

        # pack into box
        self.pack_end(self._inc_light, False, False, 0)
        self.pack_end(a_pad_left, False, False, 0)
        self.pack_end(self._inc_sat, False, False, 0)
        self.pack_end(self._dec_sat, False, False, 0)

        # adjustment controls
        self._adj = AdjustmentControls(1, 15, init_adj)
        self.pack_start(self._adj, False, False, 0)

    def _on_inc_light_clicked(self, btn):
        if self._user_on_inc_light:
            self._user_on_inc_light()

    def _on_dec_light_clicked(self, btn):
        if self._user_on_dec_light:
            self._user_on_dec_light()

    def _on_inc_sat_clicked(self, btn):
        if self._user_on_inc_sat:
            self._user_on_inc_sat()

    def _on_dec_sat_clicked(self, btn):
        if self._user_on_dec_sat:
            self._user_on_dec_sat()

    @property
    def incr_value(self):
        return self._adj.value

    def on_inc_light(self, cb):
        self._user_on_inc_light = cb

    def on_dec_light(self, cb):
        self._user_on_dec_light = cb

    def on_inc_sat(self, cb):
        self._user_on_inc_sat = cb

    def on_dec_sat(self, cb):
        self._user_on_dec_sat = cb

    def on_incr_change(self, cb):
        self._adj.on_incr_change(cb)



from delicolour.fine_controls import FineControls
from delicolour.css_hex_entry import CssHexEntry
from delicolour.css_rgb_entry import CssRgbEntry
from delicolour.fav_colours import FavColours
from delicolour.scale_entry import ScaleEntry
from delicolour.big_colour import BigColour
from delicolour.app_model import AppModel
from delicolour.colour import Colour
from gi.repository import Pango
from delicolour import config
from gi.repository import Gtk
from gi.repository import Gdk
from functools import partial


class MainWindow(Gtk.Window):
    def __init__(self, args):
        super().__init__(title='delicolour')
        self._args = args
        self._make_me_nice()
        self._init_main_box()
        self._init_keyb()
        self._model = AppModel.get_default()
        self._apply_args()
        self._update_view('init')

    def _apply_arg_init(self, init):
        try:
            colour = Colour.from_hex(init)
        except:
            return

        self._model.colour = colour

    def _apply_args(self):
        if self._args.left_colour is not None:
            try:
                colour = Colour.from_hex(self._args.left_colour)
                self._model.colour1 = colour
            except:
                pass

        if self._args.right_colour is not None:
            try:
                colour = Colour.from_hex(self._args.right_colour)
                self._model.colour2 = colour
            except:
                pass

        self._model.fine_incr = self._args.increment

    def _make_me_nice(self):
        self.set_position(Gtk.WindowPosition.CENTER)
        self.set_border_width(round(config.MAIN_GUTTER_PX * 1.5))
        self.set_resizable(False)

    def _init_keyb(self):
        self.connect('key_press_event', self._on_key_press)

    def _on_key_press(self, widget, event):
        key_name = Gdk.keyval_name(event.keyval)

        if key_name == 'z':
            # set to black
            self._set_model_rgb(0, 0, 0)
        elif key_name == 'x':
            # set to white
            self._set_model_rgb(255, 255, 255)
        elif key_name == 'minus' or key_name == 'KP_Subtract':
            # decrease lightness
            self._on_dec_light()
        elif key_name == 'plus' or key_name == 'equal' or key_name == 'KP_Add':
            # increase lightness
            self._on_inc_light()
        elif key_name == 'q':
            # decrease saturation
            self._on_dec_sat()
        elif key_name == 'w':
            # increase saturation
            self._on_inc_sat()
        elif key_name == 'numbersign':
            # swap colours
            if self._model.sel == 1:
                self._model.sel = 2
            else:
                self._model.sel = 1

            self._update_view()

    def _init_main_box(self):
        self._main_box = Gtk.Box(spacing=config.MAIN_GUTTER_PX,
                                 homogeneous=False)
        self._main_box.set_orientation(Gtk.Orientation.VERTICAL)
        self._main_box.set_size_request(320, 0)
        self.add(self._main_box)
        self._init_all()

    def _init_big_colour(self):
        self._big_colour = BigColour()
        self._big_colour.on_sel_change(self._on_sel_change)
        self._main_box.pack_start(self._big_colour, True, True, 0)

    def _init_fav_colours(self):
        fav_colours_count = self._args.fav_colours_count

        for i in range(self._args.fav_colours_rows_count):
            self._fav_colours = FavColours(fav_colours_count=fav_colours_count)
            self._fav_colours.on_fav_colour_click(self._on_fav_colour_click)
            self._main_box.pack_start(self._fav_colours, True, True, 0)

    def _init_fine_colour_controls(self):
        # fine controls
        self._fine_controls = FineControls(config.INCR_SPINNER_MIN_VAL,
                                           config.INCR_SPINNER_MAX_VAL,
                                           self._args.increment)
        self._main_box.pack_start(self._fine_controls, True, True, 0)

        # set callbacks
        self._fine_controls.on_inc_light(self._on_inc_light)
        self._fine_controls.on_dec_light(self._on_dec_light)
        self._fine_controls.on_inc_sat(self._on_inc_sat)
        self._fine_controls.on_dec_sat(self._on_dec_sat)
        self._fine_controls.on_incr_change(self._update_model_from_incr)

    def _init_colour_controls(self):
        # build controls
        self._r_ctrl = ScaleEntry('R', 0, 255, 1, 1, 0.1, 0.3)
        self._g_ctrl = ScaleEntry('G', 0, 255, 1, 0, 0.5, 0)
        self._b_ctrl = ScaleEntry('B', 0, 255, 1, 0, 0.5, 1)
        self._h_ctrl = ScaleEntry('H', 0, 359, 1, wrap=True)
        self._s_ctrl = ScaleEntry('S', 0, 100, 1)
        self._v_ctrl = ScaleEntry('V', 0, 100, 1)

        # alignments for padding
        a1 = Gtk.Alignment()
        a1.set_padding(config.MAIN_GUTTER_PX, 0, 0, 0)
        a1.add(self._r_ctrl)
        a2 = Gtk.Alignment()
        a2.set_padding(config.MAIN_GUTTER_PX, 0, 0, 0)
        a2.add(self._h_ctrl)

        # register listeners
        self._r_ctrl.on_change(self._update_model_from_rgb)
        self._g_ctrl.on_change(self._update_model_from_rgb)
        self._b_ctrl.on_change(self._update_model_from_rgb)
        self._h_ctrl.on_change(self._update_model_from_hsv)
        self._s_ctrl.on_change(self._update_model_from_hsv)
        self._v_ctrl.on_change(self._update_model_from_hsv)
        self._r_ctrl.on_key_press(partial(self._on_scale_entry_key_press, 'r'))
        self._g_ctrl.on_key_press(partial(self._on_scale_entry_key_press, 'g'))
        self._b_ctrl.on_key_press(partial(self._on_scale_entry_key_press, 'b'))

        # pack
        self._main_box.pack_start(a1, True, True, 0)
        self._main_box.pack_start(self._g_ctrl, True, True, 0)
        self._main_box.pack_start(self._b_ctrl, True, True, 0)
        self._main_box.pack_start(a2, True, True, 0)
        self._main_box.pack_start(self._s_ctrl, True, True, 0)
        self._main_box.pack_start(self._v_ctrl, True, True, 0)

    @staticmethod
    def _new_css_entry_hbox(label, entry):
        # label
        lbl = Gtk.Label()
        lbl.modify_font(Pango.FontDescription('sans-serif 8'))
        lbl.set_text(label)
        lbl.set_width_chars(4)
        lbl.set_alignment(0, 0.5)
        color = Gdk.Color(red=config.TEXT_COLOUR_R * 65535,
                          green=config.TEXT_COLOUR_G * 65535,
                          blue=config.TEXT_COLOUR_B * 65535)
        lbl.modify_fg(Gtk.StateType.NORMAL, color)

        # box
        hbox = Gtk.Box(spacing=config.MAIN_GUTTER_PX, homogeneous=False)
        hbox.props.valign = Gtk.Align.CENTER
        hbox.pack_start(lbl, False, False, 0)
        hbox.pack_start(entry, False, False, 0)

        return hbox

    @staticmethod
    def _new_bool_option_alignment(label):
        # option
        opt = Gtk.CheckButton(label=label)
        opt.modify_font(Pango.FontDescription('sans-serif 8'))
        color = Gdk.Color(red=config.TEXT_COLOUR_R * 65535,
                          green=config.TEXT_COLOUR_G * 65535,
                          blue=config.TEXT_COLOUR_B * 65535)
        opt.modify_fg(Gtk.StateType.NORMAL, color)

        # aligment for extra padding
        opt_a = Gtk.Alignment()
        opt_a.set_padding(0, 0, config.MAIN_GUTTER_PX, 0)
        opt_a.add(opt)

        return opt, opt_a

    def _init_css_entries(self):
        # hex entry
        self._css_hex_entry = CssHexEntry()
        self._css_hex_entry.on_change(self._update_model_from_css_hex)

        # RGB entry
        self._css_rgb_entry = CssRgbEntry()
        self._css_rgb_entry.on_change(self._update_model_from_css_rgb)

        # hboxes
        css_hex_hbox = MainWindow._new_css_entry_hbox('Hex:', self._css_hex_entry)
        css_rgb_hbox = MainWindow._new_css_entry_hbox('RGB:', self._css_rgb_entry)

        # add options to hex box
        self._css_hex_copy_hash_opt, a1 = MainWindow._new_bool_option_alignment('Copy #')
        self._css_hex_copy_hash_opt_toggled_handler = self._css_hex_copy_hash_opt.connect('toggled', self._on_css_hex_copy_hash_toggled)
        self._css_hex_lower_opt, a2 = MainWindow._new_bool_option_alignment('Lowercase')
        self._css_hex_lower_opt_toggled_handler = self._css_hex_lower_opt.connect('toggled', self._on_css_hex_lower_toggled)
        css_hex_hbox.pack_start(a1, False, False, 0)
        css_hex_hbox.pack_start(a2, False, False, 0)

        # alignment for hex box
        css_hex_a = Gtk.Alignment()
        css_hex_a.set_padding(config.MAIN_GUTTER_PX, 0, 0, 0)
        css_hex_a.add(css_hex_hbox)

        self._main_box.pack_start(css_hex_a, True, True, 0)
        self._main_box.pack_start(css_rgb_hbox, True, True, 0)

    def _init_all(self):
        # fav colours
        self._init_fav_colours()

        # big colour
        self._init_big_colour()

        # fine colour controls
        self._init_fine_colour_controls()

        # colour controls
        self._init_colour_controls()

        # hex
        self._init_css_entries()

    def _on_fav_colour_click(self, event, fav_colour):
        if event.button == 1:
            self._model.colour = fav_colour.colour
            self._update_view()
        elif event.button == 3:
            fav_colour.set_colour(self._model.colour.copy())
            self._update_view()

    def _on_sel_change(self, sel):
        self._model.sel = sel
        self._update_view()

    def _on_css_hex_copy_hash_toggled(self, btn):
        self._update_model_from_settings()
        self._update_view()

    def _on_css_hex_lower_toggled(self, btn):
        self._update_model_from_settings()
        self._update_view()

    def _on_inc_light(self):
        self._model.colour.inc_light(self._model.fine_incr / 100)
        self._update_view()

    def _on_dec_light(self):
        self._model.colour.dec_light(self._model.fine_incr / 100)
        self._update_view()

    def _on_inc_sat(self):
        self._model.colour.inc_sat(self._model.fine_incr / 100)
        self._update_view()

    def _on_dec_sat(self):
        self._model.colour.dec_sat(self._model.fine_incr / 100)
        self._update_view()

    def _on_scale_entry_key_press(self, origin, key_name):
        if origin == key_name:
            return

        r, g, b = self._model.colour.rgb

        if origin == 'r':
            origin_value = self._r_ctrl.value
        elif origin == 'g':
            origin_value = self._g_ctrl.value
        elif origin == 'b':
            origin_value = self._b_ctrl.value

        if key_name == 'r':
            r = origin_value
        elif key_name == 'g':
            g = origin_value
        elif key_name == 'b':
            b = origin_value

        self._set_model_rgb(r, g, b)

    def _update_model_from_incr(self):
        incr = self._fine_controls.incr_value
        self._model.fine_incr = incr
        self._update_all_incr()

    def _get_rgb_ctrl_values(self):
        r = self._r_ctrl.value
        g = self._g_ctrl.value
        b = self._b_ctrl.value

        return r, g, b

    def _get_hsv_ctrl_values(self):
        h = self._h_ctrl.value
        s = self._s_ctrl.value
        v = self._v_ctrl.value

        return h, s, v

    def _enable_hsv_events(self, en):
        self._h_ctrl.enable_on_change(en)
        self._s_ctrl.enable_on_change(en)
        self._v_ctrl.enable_on_change(en)

    def _set_model_rgb(self, r, g, b):
        self._model.colour.set_rgb(r, g, b)
        self._update_view()

    def _update_model_from_rgb(self):
        r, g, b = self._get_rgb_ctrl_values()
        self._model.colour.set_rgb(r, g, b)
        self._update_view('rgb')

    def _update_model_from_hsv(self):
        h, s, v = self._get_hsv_ctrl_values()
        s /= 100
        v /= 100
        self._model.colour.set_hsv(h, s, v)
        self._update_view('hsv')

    def _update_model_from_css_hex(self):
        colour = self._css_hex_entry.colour
        self._model.colour = colour
        self._update_view('css-hex')

    def _update_model_from_css_rgb(self):
        colour = self._css_rgb_entry.colour
        self._model.colour = colour
        self._update_view('css-rgb')

    def _update_model_from_settings(self):
        self._model.css_hex_copy_hash = self._css_hex_copy_hash_opt.get_active()
        self._model.css_hex_lower = self._css_hex_lower_opt.get_active()
        self._update_settings()

    def _update_big_colour(self):
        self._big_colour.set_colour1(self._model.colour1)
        self._big_colour.set_colour2(self._model.colour2)
        self._big_colour.set_sel(self._model.sel)

    def _update_rgb_ctrls(self):
        r, g, b = self._model.colour.rgb
        self._r_ctrl.set_value_no_emit(r)
        self._g_ctrl.set_value_no_emit(g)
        self._b_ctrl.set_value_no_emit(b)

    def _update_hsv_ctrls(self):
        h, s, v = self._model.colour.hsv
        self._h_ctrl.set_value_no_emit(h)
        self._s_ctrl.set_value_no_emit(s * 100)
        self._v_ctrl.set_value_no_emit(v * 100)

    def _update_css_hex(self):
        self._css_hex_entry.set_colour_no_emit(self._model.colour)

    def _update_css_rgb(self):
        self._css_rgb_entry.set_colour_no_emit(self._model.colour)

    def _update_settings(self):
        self._css_hex_copy_hash_opt.handler_block(self._css_hex_copy_hash_opt_toggled_handler)
        self._css_hex_lower_opt.handler_block(self._css_hex_lower_opt_toggled_handler)
        self._css_hex_copy_hash_opt.set_active(self._model.css_hex_copy_hash)
        self._css_hex_lower_opt.set_active(self._model.css_hex_lower)
        self._css_hex_copy_hash_opt.handler_unblock(self._css_hex_copy_hash_opt_toggled_handler)
        self._css_hex_lower_opt.handler_unblock(self._css_hex_lower_opt_toggled_handler)
        self._css_hex_entry.set_copy_hash(self._model.css_hex_copy_hash)
        self._css_hex_entry.set_lower(self._model.css_hex_lower)

    def _update_all_incr(self):
        self._r_ctrl.set_page_incr(self._model.fine_incr)
        self._g_ctrl.set_page_incr(self._model.fine_incr)
        self._b_ctrl.set_page_incr(self._model.fine_incr)
        self._h_ctrl.set_page_incr(self._model.fine_incr)
        self._s_ctrl.set_page_incr(self._model.fine_incr)
        self._v_ctrl.set_page_incr(self._model.fine_incr)

    def _update_title(self):
        def get_color_title(colour, sel_colour):
            if colour is sel_colour:
                fmt = '[#{}]'
            else:
                fmt = '#{}'

            hx = colour.hex

            if not self._model.css_hex_lower:
                hx = hx.upper()

            return fmt.format(hx)

        fmt = 'delicolour: {} {}'
        colour1_title = get_color_title(self._model.colour1, self._model.colour)
        colour2_title = get_color_title(self._model.colour2, self._model.colour)
        title = fmt.format(colour1_title, colour2_title)
        self.set_title(title)

    def _update_view(self, focused_ctrl=None):
        self._update_title()
        self._update_big_colour()

        if focused_ctrl != 'hsv':
            self._update_hsv_ctrls()

        if focused_ctrl != 'rgb':
            self._update_rgb_ctrls()

        if focused_ctrl != 'css-hex':
            self._update_css_hex()

        if focused_ctrl != 'css-rgb':
            self._update_css_rgb()

        if focused_ctrl == 'init':
            self._update_settings()
            self._update_all_incr()



#!/usr/bin/env python
import sys
from setuptools import setup


# make sure we run Python 3+ here
if sys.version_info.major < 3:
    sys.stderr.write('Sorry, eboxbw needs Python 3\n')
    sys.exit(1)


import delicolour


setup(name='delicolour',
      version=delicolour.__version__,
      description='colour finder',
      author='Philippe Proulx',
      author_email='eeppeliteloop@gmail.com',
      url='https://github.com/eepp/delicolour',
      packages=['delicolour'],
      package_data={
          'delicolour': [
              'res/*.png',
          ],
      },
      install_requires=[
          'colormath>=2,<3',
      ],
      entry_points={
          'gui_scripts': [
              'delicolour = delicolour.main:run'
          ]
      })
